import pytest

import lzallright


@pytest.fixture
def lorem(request):
    return (request.session.path / "benches/lorem.txt").read_bytes()


@pytest.mark.parametrize("size_hint", [None, 100, 128 * 2**10])
def test_roundtrip(lorem, size_hint):
    c = lzallright.LZOCompressor()
    comp = c.compress(lorem)

    assert (
        lzallright.LZOCompressor.decompress(comp, output_size_hint=size_hint) == lorem
    )


def test_decompress_partial(lorem):
    c = lzallright.LZOCompressor()
    comp = c.compress(lorem)

    with pytest.raises(lzallright.InputNotConsumed) as exc:
        lzallright.LZOCompressor.decompress(comp + b"foobar")

    assert exc.value.args == (lzallright.EResult.InputNotConsumed, lorem)
    assert issubclass(exc.type, lzallright.LZOError)


def test_decompress_error(lorem):
    with pytest.raises(lzallright.LZOError) as exc:
        lzallright.LZOCompressor.decompress(lorem)

    assert exc.value.args == (lzallright.EResult.LookbehindOverrun,)



from array import array
from mmap import mmap
from typing import Optional, Tuple, Union

_BufferType = Union[array[int], bytes, bytearray, memoryview, mmap]

class LZOCompressor:
    """Object containing the compressor state.

    Thread safety:
        It is not allowed to pass instances of this class between threads.
    """

    def compress(self, data: _BufferType) -> bytes:
        """Compresses data.

        Subsequent invocations of this method reuses the compression state.  In other
        words, the total output size doesn't change if you call this method using one
        big buffer or multiple small ones.

        Args:
            data (bytes): Any python object that implements the buffer protocol

        Note:
            The GIL (Global Interpreter Lock) is released for the duration of this
            function.
        """
    @staticmethod
    def decompress(data: _BufferType, output_size_hint: Optional[int] = None) -> bytes:
        """Decompresses data.

        Args:
            data (bytes): Any python object that implements the buffer protocol
            output_size_hint: Preallocate output buffer to this size.
                Helps reducing memory overhead if decompressed size is known in advance.

        Note:
            The GIL (Global Interpreter Lock) is released for the duration of this
            function.
        """

class EResult:
    """Error codes in [`LZOError.args[0]`][lzallright._lzallright.LZOError.args]."""

    LookbehindOverrun = ...
    "Invalid input"
    OutputOverrun = ...
    """Buffer is not long enough to hold output.

    Warning:
        lzallright always tries to find the appropriate buffer size.
        If you see this, it is an error.
    """
    InputOverrun = ...
    "Input to decompress is truncated"
    Error = ...
    "Other error occurred"
    InputNotConsumed = ...
    "See [`InputNotConsumed`][lzallright._lzallright.InputNotConsumed]"

class LZOError(Exception):
    """Fatal error during compression/decompression."""

    args: Tuple[EResult]
    """Error reason.

    See [`EResult`][lzallright._lzallright.EResult]"""

class InputNotConsumed(LZOError):  # noqa: N818
    """Decompression finished with leftover data."""

    args: Tuple[EResult, bytes]
    """Error reason, with decompressed data

    ``(EResult.InputNotConsumed, decompressed: bytes)``
    """



"""lzalright LZO compression library.

A Python 3.8+ binding for [LZ👌](https://github.com/jackoalan/lzokay) library which is

> A minimal, C++14 implementation of the
> [LZO compression format](http://www.oberhumer.com/opensource/lzo/).
"""

from lzallright._lzallright import EResult, InputNotConsumed, LZOCompressor, LZOError

__all__ = [
    "EResult",
    "InputNotConsumed",
    "LZOCompressor",
    "LZOError",
]
